      subroutine fillmat3 ( mat, nx, ny, nz, ex, ey, ez, bv, 
     .  sl1, sl2, sl3, sl4, sl5, sl6)
c ======================================================================
c
c
c        programmer     Jos van Kan
c        version 1.0    date   
c
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c ********************************************************************
c
c                         DESCRIPTION
c
c ********************************************************************
c
c                      KEYWORDS
c
c ********************************************************************
c
c         MODULES USED
c
      use mgpreset
c
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer bv, sl1, sl2, sl3, sl4, sl5, sl6
      integer nx, ny, nz
      double precision 
     . mat (1-sl1:nx+sl2, 1-sl3:ny+sl4, 
     . 1-sl5:nz+sl6, 27)
     . ,ex, ey, ez
c   mat             the 9 x nx x ny coefficient matrix of the
c                   discrete Laplacian
c   nx              number of grid x points on finest grid
c   ny              number of grid y points on finest grid
c	sl1	i    Slack variable, x lower bound
c	sl2	i    Slack variable, x upper bound
c	sl3	i    Slack variable, y lower bound
c	sl4	i    Slack variable, y upper bound
c	sl5	i    Slack variable, z lower bound
c	sl6	i    Slack variable, z upper bound
c		    zhi)
c
c ********************************************************************
c
c                      COMMON BLOCKS
c
c
c ********************************************************************
c
c                      LOCAL PARAMETERS
      integer ix, iy, iz, ip
      double precision cx, cy, cz, dx, dy, dz
c
c   ix              counter to number of grid x points on finest grid
c   iy              counter to number of grid y points on finest grid
c
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c     
c ********************************************************************
c
c                      I/O
c
c ********************************************************************
c
c                      ERROR MESSAGES
c
c ********************************************************************
c
c                      PSEUDO CODE
c
c ======================================================================
      dx = 1d0/(nx + 1)
      dy = 1d0/(ny + 1)
      dz = 1d0/(nz + 1)
      cx = ex * dz * dy/dx
      cy = ey * dz * dx/dy
      cz = ez * dy * dx/dz
      do iz = 1, nz
        do iy = 1, ny
          do ix = 1, nx
             mat (ix, iy, iz, p100) = 0d0
             mat (ix, iy, iz, p122) = 0d0
             mat (ix, iy, iz, p120) = 0d0
             mat (ix, iy, iz, p102) = 0d0
             mat (ix, iy, iz, p010) = 0d0
             mat (ix, iy, iz, p212) = 0d0
             mat (ix, iy, iz, p210) = 0d0
             mat (ix, iy, iz, p012) = 0d0
             mat (ix, iy, iz, p001) = 0d0
             mat (ix, iy, iz, p221) = 0d0
             mat (ix, iy, iz, p201) = 0d0
             mat (ix, iy, iz, p021) = 0d0
             mat (ix, iy, iz, p002) = 0d0
             mat (ix, iy, iz, p222) = 0d0
             mat (ix, iy, iz, p202) = 0d0
             mat (ix, iy, iz, p022) = 0d0
             mat (ix, iy, iz, p220) = 0d0
             mat (ix, iy, iz, p000) = 0d0
             mat (ix, iy, iz, p020) = 0d0
             mat (ix, iy, iz, p200) = 0d0
             mat (ix, iy, iz, p111) = 2d0 * (cx + cy + cz)
           end do
  
           do ix = 1, nx-1
             mat (ix, iy, iz, p211) = -cx
           end do
           mat (nx, iy, iz, p211) = 0d0
  
           do ix = 2, nx
             mat (ix, iy, iz, p011) = -cx
           end do
           mat (1, iy, iz, p011) = 0d0
         end do 
         
         do iy = 2, ny
           do ix = 1, nx
             mat (ix, iy, iz, p101) = -cy
           end do
         end do
         do ix = 1, nx
           mat (ix, 1, iz, p101) = 0d0
         end do
         
         do iy = 1, ny-1 
           do ix = 1, nx
             mat (ix, iy, iz, p121) = -cy
           end do
         end do
         do ix = 1, nx
           mat (ix, ny, iz, p121) = 0d0
         end do
      end do
      do iz = 2, nz
        do iy = 1, ny
          do ix = 1, nx
            mat (ix, iy, iz, p110) = -cz
          end do
        end do
      end do
      do iz = 1, nz-1
        do iy = 1, ny
          do ix = 1, nx
            mat (ix, iy, iz, p112) = -cz
          end do
        end do
      end do
      do iy = 1, ny
        do ix = 1, nx
          mat (ix, iy, 1, p110) = 0d0
          mat (ix, iy, nz, p112) = 0d0
        end do
      end do
      
c --- now zeroize on all unused space
      do ip = 1, 27
c --- upper & lower z-planes
        do iz = 1-sl5, 0
        do iy = 1-sl3, ny+sl4
        do ix = 1-sl1, nx+sl2
           mat (ix, iy, iz, ip) = 0d0
        end do
        end do
        end do

        do iz = nz+1, nz+sl6
        do iy = 1-sl3, ny+sl4
        do ix = 1-sl1, nx+sl2
           mat (ix, iy, iz, ip) = 0d0
        end do
        end do
        end do

c --  - front & back y-planes
        do iz = 1, nz
        do iy = 1-sl3, 0
        do ix = 1-sl1, nx+sl2
           mat (ix, iy, iz, ip) = 0d0
        end do
        end do
        end do

        do iz = 1, nz
        do iy = ny+1, ny+sl4
        do ix = 1-sl1, nx+sl2
           mat (ix, iy, iz, ip) = 0d0
        end do
        end do
        end do

c --  - left and right x-planes
        do iz = 1, nz
        do iy = 1, ny
        do ix = 1-sl1, 0
           mat (ix, iy, iz, ip) = 0d0
        end do
        end do
        end do

        do iz = 1, nz
        do iy = 1, ny
        do ix = nx+1, nx+sl2
           mat (ix, iy, iz, ip) = 0d0
        end do
        end do
        end do
      end do

c --- if bv = 1 then Neuman BC at the end:
      if (bv .eq. 1) then
      do iy=1, ny
      do ix=1, nx
      mat (ix, iy, nz, p111) = mat (ix, iy, nz, p111) - cz
      end do
      end do

      do iz=1, nz
      do ix=1, nx
      mat (ix, ny, iz, p111) = mat (ix, ny, iz, p111) - cy
      end do
      end do

      do iz=1, nz
      do iy=1, ny
      mat (nx, iy, iz, p111) = mat (nx, iy, iz, p111) - cx
      end do
      end do
      endif
      end
