      subroutine printmat3 ( mat, nx, ny, nz, 
     .  sl1, sl2, sl3, sl4, sl5, sl6)
c ======================================================================
c
c
c        programmer     Jos van Kan
c        version 1.0    date   11-03-97
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 2d mgpackage
c
c ********************************************************************
c
c                         DESCRIPTION
c
c  a matrix consisting of a 27-point stencil on a nx * ny*nz  grid is printed
c  per ROW of the grid, one plane per line, one stencil per
c  three lines. It is assumed that the matrix has dimension (nx, ny, nz,
c   9)
c
c
c ********************************************************************
c
c                      KEYWORDS
c
c ********************************************************************
c
c         MODULES USED
c
      use mgpreset
c
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer sl1, sl2, sl3, sl4, sl5, sl6
      integer nx, ny, nz
      double precision 
     . mat(1-sl1:nx+sl2, 1-sl3:ny+sl4,
     .     1-sl5:nz+sl6,27)
c
c   mat             the 27 x nx x ny x nz coefficient matrix of the
c                   discrete Laplacian
c   nx              number of grid x points on finest grid
c   ny              number of grid y points on finest grid
c   nz              number of grid z points on finest grid
c	sl1	i    Slack variable, x lower bound
c	sl2	i    Slack variable, x upper bound
c	sl3	i    Slack variable, y lower bound
c	sl4	i    Slack variable, y upper bound
c	sl5	i    Slack variable, z lower bound
c	sl6	i    Slack variable, z upper bound
c
c ********************************************************************
c
c                      COMMON BLOCKS
c  
c
c
c ********************************************************************
c
c                      LOCAL PARAMETERS
      integer ix, iy, iz
c
c   ix              counter to number of grid x points on finest grid
c   iy              counter to number of grid y points on finest grid
c   iz              counter to number of grid z points on finest grid
c
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c     
c ********************************************************************
c
c                      I/O
c
c ********************************************************************
c
c                      ERROR MESSAGES
c
c ********************************************************************
c
c                      PSEUDO CODE
c
c ======================================================================
      write (*, *) 'Print mat output. Matrix printed rows first, one ',
     . 'plane per 3x3 block, one stencil per 3 lines'
      write (*, '(2A)') '        LOWER PLANE                CENTER',
     .   ' PLANE                UPPER PLANE'
      do iz = 1, nz
      write (*,*) "z plane: ", iz
      do iy = 1, ny
      write (*,*) "y row: ", iy
         do ix = 1, nx
           write (*,*) "Point: ( ", ix, ", ", iy, ", ", iz, ")"
           write (*, 100) 
     . mat (ix,iy,iz,p020),mat (ix,iy,iz,p120),mat (ix,iy,iz,p220),
     . mat (ix,iy,iz,p021),mat (ix,iy,iz,p121),mat (ix,iy,iz,p221), 
     . mat (ix,iy,iz,p022),mat (ix,iy,iz,p122),mat (ix,iy,iz,p222) 
           write (*, 100) 
     . mat (ix,iy,iz,p010),mat (ix,iy,iz,p110),mat (ix,iy,iz,p210),
     . mat (ix,iy,iz,p011),mat (ix,iy,iz,p111),mat (ix,iy,iz,p211),
     . mat (ix,iy,iz,p012),mat (ix,iy,iz,p112),mat (ix,iy,iz,p212)
           write (*, 100)  
     . mat (ix,iy,iz,p000),mat (ix,iy,iz,p100),mat (ix,iy,iz,p200),
     . mat (ix,iy,iz,p001),mat (ix,iy,iz,p101),mat (ix,iy,iz,p201),
     . mat (ix,iy,iz,p002),mat (ix,iy,iz,p102),mat (ix,iy,iz,p202)
         write (*, '(1X)')
         end do
      end do
      end do
100   format (3f8.3, 3x, 3f8.3, 3x, 3f8.3)
      end
